==> mason-8.x-1.0.tar.gz: mason/src/MasonManager.php <==
<?php

namespace Drupal\mason;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\blazy\BlazyManagerBase;
use Drupal\mason\Entity\Mason;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides mason manager service.
 *
 * @todo lazyload[x], granular image styles, decent skins, stamps.
 */
class MasonManager extends BlazyManagerBase implements MasonManagerInterface, TrustedCallbackInterface {

  /**
   * The mason skin manager service.
   *
   * @var \Drupal\mason\MasonSkinManagerInterface
   */
  protected $skinManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->setSkinManager($container->get('mason.skin_manager'));
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRenderMason'];
  }

  /**
   * Returns mason skin manager service.
   */
  public function skinManager() {
    return $this->skinManager;
  }

  /**
   * Sets mason skin manager service.
   */
  public function setSkinManager(MasonSkinManagerInterface $skin_manager) {
    $this->skinManager = $skin_manager;
    return $this;
  }

  /**
   * Returns array of needed assets suitable for #attached for the given mason.
   */
  public function attach(array $attach = []) {
    $attach['blazy'] = TRUE;
    $load = parent::attach($attach);

    $load['library'][] = 'mason/load';

    if (isset($attach['skin']) && $skin = $attach['skin']) {
      $skins = $this->skinManager->getSkins();
      $provider = $skins[$skin]['provider'] ?? 'mason';
      $load['library'][] = 'mason/' . $provider . '.' . $skin;
    }

    $load['drupalSettings']['mason'] = Mason::load('default')->getOptions();

    $this->moduleHandler->alter('mason_attach', $load, $attach);
    return $load;
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $build): array {
    foreach (MasonDefault::themeProperties() as $key) {
      $build[$key] = $build[$key] ?? [];
    }

    $mason = [
      '#theme'      => 'mason',
      '#items'      => [],
      '#build'      => $build,
      '#pre_render' => [[$this, 'preRenderMason']],
      'items'       => [],
    ];

    $this->moduleHandler->alter('mason_build', $mason);
    return empty($build['items']) ? [] : $mason;
  }

  /**
   * {@inheritdoc}
   */
  public function buildItems(array $build) {
    $settings = $build['settings'];
    $items    = [];

    foreach ($build['items'] as $delta => $item) {
      if ($delta > $settings['_count'] - 1) {
        continue;
      }

      $attributes = $item['attributes'] ?? [];
      $settings   = isset($item['settings']) ? array_merge($settings, $item['settings']) : $settings;

      $content = [
        'box'     => $item['box'] ?? [],
        'caption' => $item['caption'] ?? [],
      ];

      $box = [
        '#theme'      => 'mason_box',
        '#item'       => $content,
        '#delta'      => $delta,
        '#attributes' => $attributes,
        '#settings'   => $settings,
        '#cache'      => $item['#cache'] ?? [],
      ];

      $items[] = $box;
      unset($box);
    }

    return $items;
  }

  /**
   * {@inheritdoc}
   */
  public function preRenderMason($element) {
    $build = $element['#build'];
    unset($element['#build']);

    // Build mason elements.
    $settings    = &$build['settings'];
    $settings   += MasonDefault::htmlSettings();
    $attachments = $this->attach($settings);
    $attachments = empty($build['attached']) ? $attachments : NestedArray::mergeDeep($build['attached'], $attachments);
    $cache       = $this->getCacheMetadata($build);
    $optionset   = $build['optionset'] ?: Mason::loadWithFallback($settings['optionset']);

    $settings['_count'] = count($optionset->getOption('sizes'));
    unset($build['optionset']);

    $element['#optionset'] = $optionset;
    $element['#settings']  = $settings;
    $element['#items']     = $this->buildItems($build);
    $element['#json']      = $optionset->getJson();
    $element['#attached']  = empty($element['#attached']) ? $attachments : NestedArray::mergeDeep($element['#attached'], $attachments);
    $element['#cache']     = empty($element['#cache']) ? $cache : NestedArray::mergeDeep($element['#cache'], $cache);

    unset($build);
    return $element;
  }

}

==> mason-8.x-1.1-rc1.tar.gz: mason/src/MasonManager.php <==
<?php

namespace Drupal\mason;

use Drupal\blazy\Blazy;
use Drupal\blazy\BlazyManagerBase;
use Drupal\mason\Entity\Mason;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides mason manager service.
 *
 * @todo lazyload[x], granular image styles, decent skins, stamps.
 */
class MasonManager extends BlazyManagerBase implements MasonManagerInterface {

  /**
   * The mason skin manager service.
   *
   * @var \Drupal\mason\MasonSkinManagerInterface
   */
  protected $skinManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->setSkinManager($container->get('mason.skin_manager'));
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRenderMason'];
  }

  /**
   * {@inheritdoc}
   */
  public function skinManager() {
    return $this->skinManager;
  }

  /**
   * Sets mason skin manager service.
   */
  public function setSkinManager(MasonSkinManagerInterface $skin_manager) {
    $this->skinManager = $skin_manager;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $build): array {
    foreach (MasonDefault::themeProperties() as $key => $default) {
      $k = in_array($key, ['items', 'postscript']) ? $key : "#$key";
      $build[$k] = $this->toHashtag($build, $key) ?: $default;
    }

    $mason = [
      '#theme'      => 'mason',
      '#items'      => [],
      '#build'      => $build,
      '#pre_render' => [[$this, 'preRenderMason']],
      'items'       => [],
    ];

    $this->moduleHandler->alter('mason_build', $mason);
    return empty($build['items']) ? [] : $mason;
  }

  /**
   * {@inheritdoc}
   */
  public function loadSafely($name): Mason {
    return Mason::loadSafely($name);
  }

  /**
   * {@inheritdoc}
   */
  public function preRenderMason($element) {
    $build = $element['#build'];
    unset($element['#build']);

    // Build mason elements.
    $settings = &$build['#settings'];
    $this->verify($settings);

    $optionset = Mason::verifyOptionset($build, $settings['optionset']);

    $settings['_count']    = count($optionset->getOption('sizes'));
    $element['#optionset'] = $optionset;
    $element['#settings']  = $settings;
    $element['#items']     = $this->buildItems($build);
    $element['#json']      = $optionset->getJson();

    $this->setAttachments($element, $settings);

    unset($build);
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function verify(array &$settings): void {
    // @todo use parent::verify($settings); post blazy 2.17.
    Blazy::verify($settings);

    if (!isset($settings['masons'])) {
      $settings += MasonDefault::htmlSettings();
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function attachments(array &$load, array $attach, $blazies): void {
    $this->verify($attach);

    $attach['blazy'] = TRUE;
    // @todo enable post 2.17: parent::attachments($load, $attach, $blazies);
    $load['library'][] = 'mason/load';

    if (isset($attach['skin']) && $skin = $attach['skin']) {
      $skins = $this->skinManager->getSkins();
      $provider = $skins[$skin]['provider'] ?? 'mason';
      $load['library'][] = 'mason/' . $provider . '.' . $skin;
    }

    $load['drupalSettings']['mason'] = Mason::load('default')->getOptions();

    $this->moduleHandler->alter('mason_attach', $load, $attach, $blazies);
  }

  /**
   * Returns grid items.
   */
  protected function buildItems(array $build) {
    $settings = $build['#settings'];
    $items    = [];

    foreach ($build['items'] as $delta => $item) {
      if ($delta > $settings['_count'] - 1) {
        continue;
      }

      $attrs = $this->toHashtag($item, 'attributes');
      $sets  = $this->toHashtag($item);
      $sets += $settings;

      $content = [
        'box'     => $this->toHashtag($item, 'box'),
        'caption' => $this->toHashtag($item, 'caption'),
      ];

      $box = [
        '#theme'      => 'mason_box',
        '#item'       => $content,
        '#delta'      => $delta,
        '#attributes' => $attrs,
        '#settings'   => $sets,
        '#cache'      => $item['#cache'] ?? [],
      ];

      $items[] = $box;
      unset($box);
    }

    return $items;
  }

}
